###
###  UNCLASSIFIED//FOUO
###  ARCYBER NETCOM DSD
###  JVis_v1.2.R -- computational logic and network visualization tool for JRSS service assignment
###  Author: Noah B. Jacobsen
###  Revision: 2018-12-03 Ver. 1.2 Demo
###
suppressMessages(library(shiny))
suppressMessages(library(shinythemes))

source("global.R")
source("opt.R")

JRSSINIT=jrssInit()
NODES=JRSSINIT$nodes
JRSSNAMEVEC=JRSSINIT$jrssnamevec
NUMNODES=JRSSINIT$numnodes
NUMSITES=JRSSINIT$numsites
NUMJRSS=JRSSINIT$numjrss

JRSSCODES=c("None",JRSSNAMEVEC)

## Define UI
ui <- fluidPage(
    theme = shinytheme("lumen"),

    fluidRow(
        column(
            12,
            align="center",
            textOutput(outputId = "CLASSIFICATIONTOP"),
            tags$head(tags$style("#CLASSIFICATIONTOP{color: red; font-size: 20px; }"))
            )
    ),

    fluidRow(
        column(
            1,
            imageOutput(outputId = "ARMYSTAR",height=100,width=100)
        ),
        
        column(
            11,
            textOutput(outputId = "TITLE"),
            textOutput(outputId = "JVISDESCRIPTION"),
            tags$head(tags$style("#TITLE{font-size: 26px; font-weight: bold}")),
            tags$head(tags$style("#JVISDESCRIPTION{font-size: 16px}"))
         )
    ),

    fluidRow(
        column(
            4,
            tabsetPanel(
                tabPanel(
                    "Main",
                    wellPanel(
                        selectInput(inputId = "BYPASSMODE", label = strong("Bypass Mode:"), choices = c("Full", "Partial"), selected = "Full"),
                        conditionalPanel(
                            condition = "input.BYPASSMODE == 'Full'",
                            selectInput(
                                inputId = "JRSSBYPASSFULL",
                                label = strong("JRSS Stack Full Bypass :"),
                                choices = JRSSCODES,
                                selected = "None"
                            )
                        ),
                        conditionalPanel(
                            condition = "input.BYPASSMODE == 'Partial'",
                            selectInput(
                                inputId = "JRSSBYPASSPARTIAL",
                                label = strong("JRSS Stack Partial Bypass:"),
                                choices = JRSSNAMEVEC,
                                selected = "BELVA-12"
                            ),
                            uiOutput(outputId = "PARTIALBYPASSINPUT")
                        ),
                        checkboxInput(inputId = "UTILIZATIONVIEW", label="Utilization View", value = FALSE, width = NULL),
                        actionButton(inputId = "EVALBUTTON", label = "EVAL")
                    )
                ),
                tabPanel(
                    "Capacity",
                    uiOutput(outputId = "CAPACITYSLIDERSINPUT")
                )
            )
        ),
                    
        column(
            8,
            align="center",
            tabsetPanel(
                tabPanel(
                    "Network",
                    textOutput(outputId = "ZOOMTEXT1"),
                    textOutput(outputId = "ZOOMTEXT2"),
                    tags$head(tags$style("#ZOOMTEXT1{font-size: 18px; text-align:left; font-weight:bold}")),
                    tags$head(tags$style("#ZOOMTEXT2{font-size: 16px; text-align:left}")),
                    plotOutput(outputId = "PLOT1", height = "450px", width = "600px", click = "plot1_click", dblclick = "plot1_dblclick", hover = "plot1_hover", brush = brushOpts(id="plot1_brush",resetOnNew=TRUE)),
                    imageOutput(outputId = "LEGEND1",height="160px",width="250px")
                    #verbatimTextOutput(outputId = "INFO")
                ), 
                tabPanel(
                    "Latency",
                    plotOutput(outputId = "PLOT2", height = "450px", width = "600px")
                ), 
                tabPanel(
                    "Site Data",
                    DT::dataTableOutput(outputId = "TABLE1")
                ), 
                tabPanel(
                    "Stack Data",
                    DT::dataTableOutput(outputId = "TABLE2")
                )
            )
        )
    ),
    
    fluidRow(
        column(
            12,
            align="center",
            textOutput(outputId = "FOOTER"),
            textOutput(outputId = "CLASSIFICATIONBOTTOM"),
            tags$head(tags$style("#CLASSIFICATIONBOTTOM{color: red; font-size: 20px;}"))
        )
    )
)

## Define server function
server <- function(input, output) {
    reactiveRanges <- reactiveValues(x = NULL, y = NULL)

    reactiveOpt <- reactive({
        CAPACITYCONSTRAINT=c()
        for(i in 1:NUMJRSS){
            CURRCONSTRAINT=paste0("CAPACITYCONSTRAINT", i)
            CURRCAPACITY=input[[CURRCONSTRAINT]]
            if (length(CURRCAPACITY)==0){
                CURRCAPACITY=as.numeric(sprintf("%.2f",NODES[(NUMSITES+i),COLCAPACITY]-NODES[(NUMSITES+i),COLLOAD]))
            }
            ##print(sprintf("CURRCONSTRAINT=%s",CURRCONSTRAINT))
            ##print(sprintf("CURRCAPACITY=%f",CURRCAPACITY))
            CAPACITYCONSTRAINT=c(CAPACITYCONSTRAINT,CURRCAPACITY)
        }
        jrssOpt(input$BYPASSMODE,input$JRSSBYPASSFULL,input$JRSSBYPASSPARTIAL,input$PARTIALBYPASSSELECTION,input$UTILIZATIONVIEW,JRSSINIT,CAPACITYCONSTRAINT)
    })

    reactivePlot=reactive(reactiveOpt()$netplot)
    reactiveHist=reactive(reactiveOpt()$histplot)
    reactiveSiteData=reactive(reactiveOpt()$sitedata)
    reactiveStackData=reactive(reactiveOpt()$stackdata)

    output$CAPACITYSLIDERSINPUT <- renderUI({
        lapply(1:NUMJRSS, function(i) {
            CURRCAPACITY=as.numeric(sprintf("%.2f",NODES[(NUMSITES+i),COLCAPACITY]-NODES[(NUMSITES+i),COLLOAD]))
            sliderInput(
                inputId = paste0("CAPACITYCONSTRAINT", i),
                label = paste(NODES[(NUMSITES+i),COLCODE],"Capacity Constraint (Gbps)"),
                min = 0,
                max = CURRCAPACITY,
                value = CURRCAPACITY,
                step = .01
            )
        })
    })

    output$PARTIALBYPASSINPUT <- renderUI({
        PARTIALBYPASSLIST=vector(mode = "character", length = 0)

        for(i in 1:NUMSITES){
            CURRSITE=NODES[i,COLCODE]
            CURRPRIMARY=NODES[i,COLPRIMARY]

            if(CURRPRIMARY == input$JRSSBYPASSPARTIAL){
                PARTIALBYPASSLIST=c(PARTIALBYPASSLIST,CURRSITE)
            }
        }

        checkboxGroupInput(inputId = "PARTIALBYPASSSELECTION", "Choose sites for partial bypass:", PARTIALBYPASSLIST)
    })
    
    output$ARMYSTAR <- renderImage(
    {
        filename <- normalizePath(file.path('ArmyStar.png'))
        list(src=filename,alt="ArmyStar.png",width=77,height=99)
    },
    deleteFile = FALSE
    )

    output$PLOT1 <- renderPlot({
        input$EVALBUTTON
        par(mar = c(1, 1, 1, 1))
        isolate(reactivePlot()) +
                                        #geom_point() +
            coord_cartesian(xlim = reactiveRanges$x, ylim = reactiveRanges$y, expand = TRUE)
        
    })

    output$LEGEND1 <- renderImage(
    {
        input$EVALBUTTON
        isolate({
            if(input$JRSSBYPASSFULL=="None" && input$BYPASSMODE=="Full"){
                filename <- normalizePath(file.path('legend0.png'))
            }
            else{
                filename <- normalizePath(file.path('legend1.png'))
            }
        })
        list(src=filename,alt="legend.png",width=250)
    },
    deleteFile = FALSE
    )

    output$PLOT2 <- renderPlot({
        input$EVALBUTTON
        par(mar = c(1, 1, 1, 1))
        isolate(reactiveHist())
    })

    output$TABLE1 <- DT::renderDataTable({
        input$EVALBUTTON
        isolate(DT::datatable(reactiveSiteData(),rownames=FALSE))
    })
    
    output$TABLE2 <- DT::renderDataTable({
        input$EVALBUTTON
        isolate(DT::datatable(reactiveStackData(),rownames=FALSE))
    })
    
    output$FOOTER <- renderText(paste0(VERSION,". U.S. Army NETCOM. Author: Dr. Noah B. Jacobsen"))
    output$CLASSIFICATIONTOP <- renderText(CLASSIFICATIONTEXT)
    output$CLASSIFICATIONBOTTOM <- renderText(CLASSIFICATIONTEXT)

    observeEvent(input$plot1_dblclick, {
        brush <- input$plot1_brush
        if (!is.null(brush)) {
            reactiveRanges$x <- c(brush$xmin, brush$xmax)
            reactiveRanges$y <- c(brush$ymin, brush$ymax)

        } else {
            reactiveRanges$x <- NULL
            reactiveRanges$y <- NULL
        }
    })    

    output$TITLE <- renderText(VERSION)
    output$JVISDESCRIPTION <- renderText("Computational logic and network visualization for Joint Regional Security Stack (JRSS) service management for the U.S. Government's Joint Information Environment (JIE)")
    output$ZOOMTEXT1 <- renderText("Zoom plot:")
    output$ZOOMTEXT2 <- renderText("Drag cursor to create zoom region and double-click inside box to zoom. Double-click to reset.")
        
    output$INFO <- renderText({
        xy_str <- function(e) {
            if(is.null(e)) return("NULL\n")
            paste0("x=", round(e$x, 1), " y=", round(e$y, 1), "\n")
        }
        xy_range_str <- function(e) {
            if(is.null(e)) return("NULL\n")
            paste0("xmin=", round(e$xmin, 1), " xmax=", round(e$xmax, 1), 
                   " ymin=", round(e$ymin, 1), " ymax=", round(e$ymax, 1))
        }
 
        paste0(
            "click: ", xy_str(input$plot1_click),
            "dblclick: ", xy_str(input$plot1_dblclick),
            "hover: ", xy_str(input$plot1_hover),
            "brush: ", xy_range_str(input$plot1_brush)
        )
    })
}

app <- shinyApp(ui = ui, server = server)
runApp(app,port=4550)
