###
###  UNCLASSIFIED//FOUO
###  ARCYBER NETCOM DSD
###  opt.R -- optimization engine for JVis Tool
###  Author: Noah B. Jacobsen
###  Revision: 2018-12-03 Ver. 1.2 Demo
###

jrssInit <- function(){
    suppressMessages(library(geosphere))
    source("global.R")

    NODES <- read.csv(sprintf("%s.csv",INPUTFILE), header=T, as.is=T)
    NUMNODES=nrow(NODES)

    ##get names of JRSS stacks
    JRSSNAMEVEC=vector(mode="character",length=0)
    NUMJRSS=0
    for (i in 1:NUMNODES) {
        if (NODES[i,COLTYPE]=="JRSS"){
            NUMJRSS=NUMJRSS+1
            JRSSNAMEVEC=c(JRSSNAMEVEC,NODES[i,COLCODE])
        }
    }
    NUMSITES=NUMNODES-NUMJRSS
    COLSTOPJRSSLATENCY=COLSTARTJRSSLATENCY+NUMJRSS-1

    SITELATENCY=matrix(NA,NUMSITES,NUMJRSS)
    RESP=vector(mode="numeric",length=0)
    PRED1=vector(mode="numeric",length=0)              

    ##find linear predictor of latency as a function of intersite distance
    for (i in (NUMSITES+1):NUMNODES) {
        CURRCOLLATENCY=COLSTARTJRSSLATENCY-NUMSITES-1+i
        JRSSNAME=NODES[i,COLCODE]

        CURRJRSSLAT=NODES[i,COLLATITUDE]
        CURRJRSSLONG=NODES[i,COLLONGITUDE]
        P1=c(CURRJRSSLONG,CURRJRSSLAT)
        
        for (j in 1:NUMSITES) {
            CURRLATENCY=NODES[j,CURRCOLLATENCY]

            CURRSITELAT=as.numeric(NODES[j,COLLATITUDE])
            CURRSITELONG=as.numeric(NODES[j,COLLONGITUDE])
            P2=c(CURRSITELONG,CURRSITELAT)
            SITEDISTANCE=distGeo(P1,P2)
            SITELATENCY[j,(i-NUMSITES)]=SITEDISTANCE
 
            if(!is.na(CURRLATENCY)){
                if(CURRLATENCY>0){
                    RESP=c(RESP,CURRLATENCY)
                    PRED1=c(PRED1,SITEDISTANCE)
                }
            }
        }
    }

    LM_DF=data.frame(
        resp=RESP,
        pred1=PRED1
    )

    LM_RESULT <- lm(resp~pred1,data=LM_DF)
    coeffs <- summary(LM_RESULT)$coefficients[,1]
    SITELATENCY=coeffs[2]*SITELATENCY + coeffs[1]

    ##populate unknown latencies with predicted value
    for (i in 1:NUMJRSS) {
        CURRCOLLATENCY=COLSTARTJRSSLATENCY+i-1
        
        for (j in 1:NUMSITES) {
            CURRLATENCY=NODES[j,CURRCOLLATENCY]
            
            if(is.na(CURRLATENCY)){
                NODES[j,CURRCOLLATENCY]=SITELATENCY[j,i]
            }
            else if(CURRLATENCY==0.0){
                NODES[j,CURRCOLLATENCY]=SITELATENCY[j,i]
            }
        }
    }

    RETURN=list(nodes=NODES,jrssnamevec=JRSSNAMEVEC,numnodes=NUMNODES,numsites=NUMSITES,numjrss=NUMJRSS)
    return(RETURN)
}

jrssOpt <- function(BYPASSMODE,JRSSBYPASSFULL,JRSSBYPASSPARTIAL,PARTIALBYPASSSELECTION,UTILIZATIONVIEW,JRSSINIT,CAPACITYCONSTRAINT){
    #print(sprintf("BYPASSMODE=%s",BYPASSMODE))
    #print(sprintf("JRSSBYPASSFULL=%s",JRSSBYPASSFULL))
    #print(sprintf("JRSSBYPASSPARTIAL=%s",JRSSBYPASSPARTIAL))
    #print(sprintf("PARTIALBYPASSSELECTION=%s",PARTIALBYPASSSELECTION))
    #print(sprintf("UTILIZATIONVIEW=%s",UTILIZATIONVIEW))

    suppressMessages(library(igraph))
    suppressMessages(library(lpSolve))
    suppressMessages(library(ggplot2))
    suppressMessages(library(GGally))
    suppressMessages(library(intergraph))
    suppressMessages(library(reshape2))

    GENGRAPH1=FALSE #latency histogram current topology 
    PLOTNETMAP1=FALSE #current topology
    EXELINPROG5=FALSE #optimal selection of secondary/tertiary jrss nodes for bypass scenario
    GENGRAPH5=FALSE #current topology vs optimal hard-coded jrss w/ removed node
    PLOTNETMAP5=FALSE #plot current hard-coded jrss vs optimal hard-coded w/ removed node

    if(JRSSBYPASSFULL=="None" && BYPASSMODE=="Full"){
        GENGRAPH1=TRUE 
        PLOTNETMAP1=TRUE
    }
    else{
        GENGRAPH1=TRUE
        PLOTNETMAP1=TRUE
        EXELINPROG5=TRUE
        GENGRAPH5=TRUE
        PLOTNETMAP5=TRUE
    }

    ##Add capacity constraint
    for(i in 1:NUMJRSS){
        #print(sprintf("CAPACITYCONSTRAINT[%d]=%f",i,CAPACITYCONSTRAINT[i]))
        NODES[(NUMSITES+i),COLCAPACITY]=CAPACITYCONSTRAINT[i]
    }
    
    ##Generate initial IGRAPH network model
    if(GENGRAPH1){
        INITEDGESROWS=1000
        INITEDGESCOLUMNS=3
        edges=matrix(NA,INITEDGESROWS,INITEDGESCOLUMNS)

        j=1
        for (i in 1:NUMNODES) {
            if (NODES[i,COLTYPE]=="P/C/B/S"){
                SITEID=NODES[i,1]

                if (NODES[i,COLPRIMARY]!=""){
                    JRSS=NODES[i,COLPRIMARY]
                    JRSSROWINDEX=which(NODES[,COLCODE]==JRSS, arr.ind=TRUE)
            
                    if (length(JRSSROWINDEX)!=1){
                        stop("JRSS site code search error.")
                    }
            
                    JRSSID=NODES[JRSSROWINDEX,COLID]
                    
                    edges[j,1]=SITEID
                    edges[j,2]=JRSSID
                    edges[j,3]="primary"
                    j=j+1
                }
            }
        }
        edges=edges[1:j-1,1:2]

        ##HIST1 -- hard-coded primary stacks
        LATENCYVEC1=vector(mode="numeric",length=NUMSITES)
        LOADVEC1=vector(mode="numeric",length=NUMSITES)

        for (i in 1:NUMSITES) {
            PRIMJRSS=NODES[i,COLPRIMARY]
            PRIMJRSSINDEX=which(JRSSNAMEVEC==PRIMJRSS)

            LATENCYVEC1[i]=NODES[i,(COLSTARTJRSSLATENCY-1+PRIMJRSSINDEX)]
            LOADVEC1[i]=NODES[i,COLLOAD]
        }

        HIST1=genHist(LATENCYVEC1,LOADVEC1,NUMSITES,HISTBINSIZE)
        NUMBINS1=HIST1$numbins
        LATENCYBINS1=HIST1$latencybins
        LATENCYHIST1=HIST1$latencyhist
        AVELATENCY1=HIST1$avelatency

        SITEDATA1=matrix(NA,NUMSITES,4)
        for(i in 1:NUMSITES){
            SITEDATA1[i,1]=NODES[i,COLCODE]
            SITEDATA1[i,2]=NODES[i,COLPRIMARY]
            SITEDATA1[i,3]=sprintf("%.2f",LOADVEC1[i])
            SITEDATA1[i,4]=sprintf("%.2f",LATENCYVEC1[i])
        }

        STACKDATA1=matrix(NA,NUMJRSS,4)
        JRSSUTIL1=rep(0.0,NUMJRSS)

        for(i in 1:NUMJRSS){
            CURRSTACK=NODES[(NUMSITES+i),COLCODE]
            CURRCAPACITY=NODES[(NUMSITES+i),COLCAPACITY]

            for(j in 1:NUMSITES){
                CURRPRIM=NODES[j,COLPRIMARY]

                if( CURRSTACK==CURRPRIM ){
                    JRSSUTIL1[i]=JRSSUTIL1[i]+LOADVEC1[j]
                }
            }
            
            STACKDATA1[i,1]=CURRSTACK
            STACKDATA1[i,2]=sprintf("%.2f",CURRCAPACITY)
            STACKDATA1[i,3]=sprintf("%.2f",JRSSUTIL1[i])
            STACKDATA1[i,4]=sprintf("%.1f",(100*(JRSSUTIL1[i]/CURRCAPACITY)))
        }
    }

    ##plot network graph of all primary JRSS links
    if ( PLOTNETMAP1 ){
        IGRAPH=graph_from_data_frame(d=edges,vertices=NODES,directed=FALSE)
        
        PCBSNODE=(V(IGRAPH)$type=="P/C/B/S")
        JRSSNODE=(V(IGRAPH)$type=="JRSS")
    
        NODELABELCOLOR=vector(mode="character",length=NUMNODES)
        NODELABELCOLOR[PCBSNODE]=PCBSLABELCOLOR
        NODELABELCOLOR[JRSSNODE]=JRSSLABELCOLOR

        NODELABELSIZE=vector(mode="numeric",length=NUMNODES)
        NODELABELSIZE[PCBSNODE]=PCBSLABELSIZE
        NODELABELSIZE[JRSSNODE]=JRSSLABELSIZE

        NODECOLOR=vector(mode="character",length=NUMNODES)
        NODECOLOR[PCBSNODE]=PCBSCOLOR
        NODECOLOR[JRSSNODE]=JRSSCOLOR
    
        NODESIZE=vector(mode="numeric",length=NUMNODES)
        NODESIZE[PCBSNODE]=PCBSNODESIZE
        NODESIZE[JRSSNODE]=JRSSNODESIZE

        if( UTILIZATIONVIEW ){
            NODELABEL=vector(mode="character",length=NUMNODES)
            for (i in 1:NUMSITES) {
                NODELABEL[i]=sprintf("%s: %0.2f",NODES[i,COLCODE],NODES[i,COLLOAD])
            }
            
            for (i in 1:NUMJRSS) {
                NODELABEL[(NUMSITES+i)]=sprintf("%s: %0.2f/%0.2f",NODES[(NUMSITES+i),COLCODE],JRSSUTIL1[i],NODES[(NUMSITES+i),COLCAPACITY])
            }
        }
        
        else{
            NODELABEL=NODES$code
        }

        NUMEDGES=nrow(edges)
        EDGECOLOR=rep(PRIMARYEDGECOLOR,times=NUMEDGES)
        EDGEWIDTH=rep(PRIMARYEDGEWIDTH,times=NUMEDGES)
        EDGESTYLE=rep(PRIMARYEDGESTYLE,times=NUMEDGES)

        NGRAPH <- asNetwork(IGRAPH,amap=attrmap())
        #NGRAPH %e% "connection" = "Primary"
        #network::set.edge.attribute(NGRAPH, "connection", "Primary")
        #print(NGRAPH)
        
        NETPLOT<-ggnet2(
            NGRAPH,
            color="type",
            palette=c("P/C/B/S"=PCBSCOLOR,"JRSS"=JRSSCOLOR),
            label=NODELABEL,
            size="type",
            size.palette=c("P/C/B/S"=PCBSNODESIZE,"JRSS"=JRSSNODESIZE),
            label.size=NODELABELSIZE,
            label.color=NODELABELCOLOR,
            edge.color=EDGECOLOR,
            edge.size=EDGEWIDTH,
            edge.lty=EDGESTYLE,
            node.shape=NODESHAPE
        ) +
            guides(color=FALSE,size=FALSE)
    }

    ##linear program to compute optimal hard-coded selection of JRSS vs. bypass
    if( EXELINPROG5 ){
        SITEJRSSMAP5=matrix(NA,NUMSITES,4)
        NUMVARIABLES5=0
    
        for (i in 1:NUMSITES) {
            SITECODE=NODES[i,COLCODE]

            NUMAVAILJRSS=0
            for (j in 1:3){
                JRSSCODE=NODES[i,(COLPRIMARY-1+j)]

                if (BYPASSMODE == "Full"){
                    if ( (JRSSCODE != "")  && (JRSSCODE != JRSSBYPASSFULL) ) {
                        NUMAVAILJRSS=NUMAVAILJRSS+1
                        JRSSINDEX=which(JRSSNAMEVEC==JRSSCODE)
                        
                        SITEJRSSMAP5[i,(NUMAVAILJRSS+1)]=JRSSINDEX
                    }
                }
                else if (BYPASSMODE == "Partial"){
           
                    if (JRSSCODE == JRSSBYPASSPARTIAL){
                        ISPARTIALBYPASS=sum(PARTIALBYPASSSELECTION==SITECODE)>0
                    }
                    else{
                        ISPARTIALBYPASS=FALSE
                    }
                    
                    if ( (JRSSCODE != "")  && (!ISPARTIALBYPASS) ) {
                        NUMAVAILJRSS=NUMAVAILJRSS+1
                        JRSSINDEX=which(JRSSNAMEVEC==JRSSCODE)
                        
                        SITEJRSSMAP5[i,(NUMAVAILJRSS+1)]=JRSSINDEX
                    }
                }
            }
            SITEJRSSMAP5[i,1]=NUMAVAILJRSS
            NUMVARIABLES5=NUMVARIABLES5+NUMAVAILJRSS
        }

        OBJECTIVEFN=vector(mode="numeric",length=NUMVARIABLES5)
        ##CONSTRAINTMATRIX=matrix(0,NUMSITES+NUMJRSS+NUMVARIABLES5,NUMVARIABLES5)
        CONSTRAINTMATRIX=matrix(0,NUMSITES+NUMJRSS,NUMVARIABLES5)
        
        COLSTARTINDEX=1
        for (i in 1:NUMSITES) {

            NUMAVAILJRSS=SITEJRSSMAP5[i,1]
            for (j in 1:NUMAVAILJRSS){
                JRSSINDEX=SITEJRSSMAP5[i,(j+1)]
                
                SITELATENCY=NODES[i,(COLSTARTJRSSLATENCY+JRSSINDEX-1)]
                SITELOADREQ=NODES[i,COLLOAD]
                WEIGHTEDLATENCY=(SITELATENCY*SITELOADREQ)
                
                CURRCOLINDEX=(COLSTARTINDEX-1+j)
                CURRROWINDEX=(NUMSITES+JRSSINDEX)
                CURRROWINDEX2=(NUMNODES+CURRCOLINDEX)
                
                OBJECTIVEFN[CURRCOLINDEX]=WEIGHTEDLATENCY
            
                CONSTRAINTMATRIX[i,CURRCOLINDEX]=1
                CONSTRAINTMATRIX[CURRROWINDEX,CURRCOLINDEX]=SITELOADREQ
                ##CONSTRAINTMATRIX[CURRROWINDEX2,CURRCOLINDEX]=SITELATENCYDATA
            }
            COLSTARTINDEX=COLSTARTINDEX+NUMAVAILJRSS
        }

        CONSTRAINTDIR1=rep("=",NUMSITES)
        ##CONSTRAINTDIR2=rep("<=",NUMJRSS+NUMVARIABLES5)
        CONSTRAINTDIR2=rep("<=",NUMJRSS)
        CONSTRAINTDIR=c(CONSTRAINTDIR1,CONSTRAINTDIR2)

        INDICATORSELECTREQ=rep(1,NUMSITES)
        JRSSCAPACITYBOUND=as.numeric(NODES[(NUMSITES+1):NUMNODES,COLCAPACITY])
        MAXDELAYCONSTRAINT=rep(100,NUMVARIABLES5)

        ##CONSTRAINTRHS=c(INDICATORSELECTREQ,JRSSCAPACITYBOUND,MAXDELAYCONSTRAINT)
        CONSTRAINTRHS=c(INDICATORSELECTREQ,JRSSCAPACITYBOUND)

        RESULT=lp(
            direction="min",
            OBJECTIVEFN,
            CONSTRAINTMATRIX,
            CONSTRAINTDIR,
            CONSTRAINTRHS,
            transpose.constraints=TRUE,
            all.int=TRUE,
            )
        
        STATUS=RESULT$status
    
        ##if ( STATUS == 2 ) {
            ##print("*******************************")
            ##print(sprintf("STATUS=%s: No feasible solution (EXELINPROG5).",STATUS))        
            ##print("*******************************")
        ##}
        ##else{
            ##print(sprintf("STATUS=%s: Feasible solution found (EXELINPROG5).",STATUS))        
        ##}
            
        SOLUTION5=RESULT$solution        
        SOLUTIONCONSTRAINTS=CONSTRAINTMATRIX %*% SOLUTION5
        JRSSUTIL5=SOLUTIONCONSTRAINTS[(NUMSITES+1):(NUMSITES+NUMJRSS)]
        OBJVAL=RESULT$objval
    }

    ##current hard-coded JRSS vs. optimal selection of hard-coded JRSS with site bypass
    if ( GENGRAPH5 ){
        edges5=matrix(NA,NUMSITES,3)
        colnames(edges5) <- c("node0","node1","type")

        PRIMJRSS5=rep(NA,NUMSITES)
        
        NUMALTPRIMARY5=0
        COLSTARTINDEX5=1

        LATENCYVEC5=vector(mode="numeric",length=NUMSITES)
        LOADVEC5=vector(mode="numeric",length=NUMSITES)
        
        for (i in 1:NUMSITES) {
            NUMAVAILJRSS5=SITEJRSSMAP5[i,1]
            CURSELECTVEC=SOLUTION5[COLSTARTINDEX5:(COLSTARTINDEX5+NUMAVAILJRSS5-1)]        
            CURSELECTIND=which(CURSELECTVEC==1)
            CURJRSSIND=SITEJRSSMAP5[i,(CURSELECTIND+1)]
            
            PRIMJRSS5[i]=JRSSNAMEVEC[CURJRSSIND]
            
            LATENCYVEC5[i]=NODES[i,(COLSTARTJRSSLATENCY-1+CURJRSSIND)]
            LOADVEC5[i]=NODES[i,COLLOAD]
            
            PRIMJRSS=NODES[i,COLPRIMARY]
            PRIMJRSSIND=which(JRSSNAMEVEC==PRIMJRSS)

            edges5[i,1]=NODES[i,COLID]
            edges5[i,2]=NODES[(NUMSITES+PRIMJRSSIND),COLID]
            edges5[i,3]="primary"
        
            if ( PRIMJRSSIND != CURJRSSIND ) {           
                NUMALTPRIMARY5=NUMALTPRIMARY5+1
                
                NODEID1=NODES[i,COLID]
                NODEID2=NODES[NUMSITES+CURJRSSIND,COLID]
                
                NEWEDGE=c(NODEID1,NODEID2,"alt_primary")
                edges5=rbind(edges5,NEWEDGE)
            }

            COLSTARTINDEX5=COLSTARTINDEX5+NUMAVAILJRSS5
        }
    
        HIST5=genHist(LATENCYVEC5,LOADVEC5,NUMSITES,HISTBINSIZE)
        NUMBINS5=HIST5$numbins
        LATENCYBINS5=HIST5$latencybins
        LATENCYHIST5=HIST5$latencyhist
        AVELATENCY5=HIST5$avelatency

        SITEDATA5=matrix(NA,NUMSITES,2)
        for(i in 1:NUMSITES){
            SITEDATA5[i,1]=PRIMJRSS5[i]
            SITEDATA5[i,2]=sprintf("%.2f",LATENCYVEC5[i])
        }

        STACKDATA5=matrix(NA,NUMJRSS,2)
        for(i in 1:NUMJRSS){
            CURRCAPACITY=NODES[(NUMSITES+i),COLCAPACITY]

            STACKDATA5[i,1]=sprintf("%.2f",JRSSUTIL5[i])
            STACKDATA5[i,2]=sprintf("%.1f",(100*(JRSSUTIL5[i]/CURRCAPACITY)))            
        }
    }

    ##Optimal Selection of Hard-Coded JRSS Stack vs. Bypass
    if ( PLOTNETMAP5 ){
        IGRAPH=graph_from_data_frame(d=edges5,vertices=NODES,directed=T)
        
        PCBSNODE=(V(IGRAPH)$type=="P/C/B/S")
        JRSSNODE=(V(IGRAPH)$type=="JRSS")
    
        NODELABELCOLOR=vector(mode="character",length=NUMNODES)
        NODELABELCOLOR[PCBSNODE]=PCBSLABELCOLOR
        NODELABELCOLOR[JRSSNODE]=JRSSLABELCOLOR

        NODELABELSIZE=vector(mode="numeric",length=NUMNODES)
        NODELABELSIZE[PCBSNODE]=PCBSLABELSIZE
        NODELABELSIZE[JRSSNODE]=JRSSLABELSIZE

        if (BYPASSMODE=="Full"){
            JRSSBYPASSINDEX=which(JRSSNAMEVEC==JRSSBYPASSFULL)
        }
        else if(BYPASSMODE=="Partial"){
            JRSSBYPASSINDEX=which(JRSSNAMEVEC==JRSSBYPASSPARTIAL)
        }

        NODECOLOR=vector(mode="character",length=NUMNODES)
        NODECOLOR[PCBSNODE]=PCBSCOLOR
        NODECOLOR[JRSSNODE]=JRSSCOLOR
        NODECOLOR[NUMSITES+JRSSBYPASSINDEX]=JRSSRMCOLOR
    
        NODESIZE=vector(mode="numeric",length=NUMNODES)
        NODESIZE[PCBSNODE]=PCBSNODESIZE
        NODESIZE[JRSSNODE]=JRSSNODESIZE

        if( UTILIZATIONVIEW ){
            NODELABEL=vector(mode="character",length=NUMNODES)
            for (i in 1:NUMSITES) {
                NODELABEL[i]=sprintf("%s: %0.2f",NODES[i,COLCODE],NODES[i,COLLOAD])
            }

            for (i in 1:NUMJRSS) {            
                NODELABEL[(NUMSITES+i)]=sprintf("%s: %0.2f/%0.2f",NODES[(NUMSITES+i),COLCODE],JRSSUTIL5[i],NODES[(NUMSITES+i),COLCAPACITY])
            }
        }
        else{
            NODELABEL=NODES$code
        }
    
        PRIMEDGE=(E(IGRAPH)$type=="primary")
        ALTPRIMEDGE=(E(IGRAPH)$type=="alt_primary")
    
        NUMEDGES=nrow(edges5)
        
        EDGECOLOR=vector(mode="character",length=NUMEDGES)
        EDGECOLOR[PRIMEDGE]=PRIMARYEDGECOLOR
        EDGECOLOR[ALTPRIMEDGE]=ALTEDGECOLOR

        EDGEWIDTH=vector(mode="numeric",length=NUMEDGES)
        EDGEWIDTH[PRIMEDGE]=PRIMARYEDGEWIDTH
        EDGEWIDTH[ALTPRIMEDGE]=ALTPRIMEDGEWIDTH
        
        EDGESTYLE=vector(mode="numeric",length=NUMEDGES)
        EDGESTYLE[PRIMEDGE]=PRIMARYEDGESTYLE
        EDGESTYLE[ALTPRIMEDGE]=ALTPRIMEDGESTYLE

        ##plot current hardcoded jrss vs optimal hard-coded jrss with removed node
        NETPLOT<-ggnet2(
            IGRAPH,
            node.color=NODECOLOR,
            node.label=NODELABEL,
            node.size=NODESIZE,
            node.shape=NODESHAPE,
            label.size=NODELABELSIZE,
            label.color=NODELABELCOLOR,
            edge.color=EDGECOLOR,
            edge.size=EDGEWIDTH,
            edge.lty=EDGESTYLE,
            color.legend="Nodes",
            legend.position="none"
        )
    }

    ##HISTPLOT and TABLES DATA output
    if(JRSSBYPASSFULL=="None" && BYPASSMODE=="Full"){
        SITEDATA_DF <- data.frame(
            "Site Code"=SITEDATA1[,1],
            "Primary JRSS"=SITEDATA1[,2],
            "Peak Site Load (Gbps)"=as.numeric(SITEDATA1[,3]),
            "Site Latency (msec)"=as.numeric(SITEDATA1[,4]),
            check.names=FALSE
        )
        
        STACKDATA_DF <- data.frame(
            "JRSS Code"=STACKDATA1[,1],
            "Stack Capacity (Gbps)"=as.numeric(STACKDATA1[,2]),
            "Peak Utilization (Gbps)"=as.numeric(STACKDATA1[,3]),
            "Peak Utilization (Percent)"=as.numeric(STACKDATA1[,4]),
            check.names=FALSE
        )
        
        histdata <- data.frame(
            bins=LATENCYBINS1,
            count1=LATENCYHIST1
        )

        histdata.m <- melt(histdata, id.vars='bins')
        
        HISTPLOT=ggplot(histdata.m,aes(bins,value)) +
            geom_bar(
                aes(fill=variable),
                width=HISTBINSIZE,
                stat="identity",
                position=position_dodge(width=HISTBINSIZE),
                color="black"
            ) +
            scale_fill_manual(
                name=NULL,
                label=sprintf("Current topology (Ave Latency=%0.2f msec)",AVELATENCY1),
                values=HISTCOLOR1
            ) +
            labs(
                x="Latency (msec)",
                y="Volume (Gbps)",
                title="Latency Histogram of Current Topology"
            ) +
            theme(
                #panel.border = element_rect(colour = "black", fill=NA),
                legend.box.background =  element_rect(linetype = 1, size = 1, colour = 1),
                #legend.position = c(HISTLEGENDPOSITIONX,HISTLEGENDPOSITIONY),
                legend.position = "bottom",
                legend.text=element_text(size=HISTLEGENDFONTSIZE),
                text=element_text(size=HISTAXISFONTSIZE),
                plot.title=element_text(hjust=0.5)
            )
    }
    
    else{
        ##Current hard-coded JRSS vs Optimal Selection of Hard-Coded JRSS Stack with Bypass
        SITEDATA_DF <- data.frame(
            "Site Code"=SITEDATA1[,1],
            "Current Primary JRSS"=SITEDATA1[,2],
            "Alternate Primary JRSS"=SITEDATA5[,1],
            "Peak Site Load (Gbps)"=as.numeric(SITEDATA1[,3]),
            "Original Site Latency (msec)"=as.numeric(SITEDATA1[,4]),
            "Alternate Site Latency (msec)"=as.numeric(SITEDATA5[,2]),
            check.names=FALSE
        )
        
        STACKDATA_DF <- data.frame(
            "JRSS Code"=STACKDATA1[,1],
            "Stack Capacity (Gbps)"=as.numeric(STACKDATA1[,2]),
            "Original Peak Utilization (Gbps)"=as.numeric(STACKDATA1[,3]),
            "Alternative Peak Utilization (Gbps)"=as.numeric(STACKDATA5[,1]),
            "Original Peak Utilization (Percent)"=as.numeric(STACKDATA1[,4]),
            "Alternative Peak Utilization (Percent)"=as.numeric(STACKDATA5[,2]),
            check.names=FALSE
        )
        
        if ( NUMBINS5>NUMBINS1 ){
            length(LATENCYHIST1) <- NUMBINS5
            LATENCYHIST1[is.na(LATENCYHIST1)] <- 0.0 
            LATENCYBINS <- LATENCYBINS5
        }
        if ( NUMBINS5<=NUMBINS1 ){
            length(LATENCYHIST5) <- NUMBINS1
            LATENCYHIST5[is.na(LATENCYHIST5)] <- 0.0 
            LATENCYBINS <- LATENCYBINS1
        }
        
        histdata <- data.frame(
            bins=LATENCYBINS,
            count1=LATENCYHIST1,
            count2=LATENCYHIST5
        )

        histdata.m <- melt(histdata, id.vars='bins')

        if (BYPASSMODE=="Full"){
            BYPASSTEXT=sprintf("%s full",JRSSBYPASSFULL)
        }
        else{
            BYPASSTEXT=sprintf("%s partial",JRSSBYPASSPARTIAL)
        }

        HISTPLOT=ggplot(histdata.m,aes(bins,value)) +
            geom_bar(
                width=HISTBINSIZE,
                aes(fill=variable),
                position=position_dodge(width=HISTBINSIZE),
                stat="identity",
                color="black"
            ) +
            scale_fill_manual(
                name=NULL,
                label=c(sprintf("Current topology (Ave Latency=%0.2f msec)",AVELATENCY1),sprintf("Optimal JRSS with %s bypass (Ave Latency=%0.2f msec)",BYPASSTEXT,AVELATENCY5)),
                values=c(HISTCOLOR1,HISTCOLOR2)
            ) +
            labs(
                x="Latency (msec)",
                y="Volume (Gbps)",
                title="Current Topology vs. Optimal JRSS with Bypass"
            ) +
            theme(
                #panel.border = element_rect(colour = "black", fill=NA),
                legend.box.background =  element_rect(linetype = 1, size = 1, colour = 1),
                legend.spacing.y = unit(0, "mm"),
                #legend.position = c(HISTLEGENDPOSITIONX,HISTLEGENDPOSITIONY),
                legend.position = "bottom",
                legend.text=element_text(size=HISTLEGENDFONTSIZE),
                text=element_text(size=HISTAXISFONTSIZE),
                plot.title=element_text(hjust=0.5)
            ) +
            guides(fill=guide_legend(nrow=2))
    }

    RESULT <- list(netplot=NETPLOT,histplot=HISTPLOT,sitedata=SITEDATA_DF,stackdata=STACKDATA_DF)
    return(RESULT)
}

genHist <- function(LATENCYVEC,LOADVEC,NUMSITES,HISTBINSIZE){
    
    MAXLATENCY=max(LATENCYVEC)
    NUMBINS=ceiling(MAXLATENCY/HISTBINSIZE)+1

    LATENCYBINS=seq((HISTBINSIZE/2),((NUMBINS-0.5)*HISTBINSIZE),HISTBINSIZE)
    LATENCYHIST=rep(0,NUMBINS)

    AVELATENCY=0
    AVELATENCYDENOM=0
    
    for (i in 1:NUMSITES){
        CURRLATENCY=LATENCYVEC[i]
        CURRLOAD=LOADVEC[i]

        AVELATENCY=AVELATENCY+CURRLATENCY*CURRLOAD
        AVELATENCYDENOM=AVELATENCYDENOM+CURRLOAD
        
        for(j in 1:NUMBINS){
            BINLOW=(j-1)*HISTBINSIZE
            BINHIGH=j*HISTBINSIZE
        
            if( (CURRLATENCY>=BINLOW) && (CURRLATENCY<BINHIGH) ){
                LATENCYHIST[j]=LATENCYHIST[j]+CURRLOAD
            }
        }
    }

    AVELATENCY=AVELATENCY/AVELATENCYDENOM

    RETURN=list(numbins=NUMBINS,latencybins=LATENCYBINS,latencyhist=LATENCYHIST,avelatency=AVELATENCY)
    return(RETURN)
}
